@@ -150,3 +150,28 @@ async def test_canceled_inside_acquire(loop):
         pass
 
     assert not rl.locked
+
+
+@pytest.mark.asyncio
+async def test_race_multiple_writers(loop):
+    seq = []
+
+    async def write_wait(lock):
+        async with lock.reader:
+            await asyncio.sleep(0.1)
+            seq.append('READ')
+        async with lock.writer:
+            seq.append('START1')
+            await asyncio.sleep(0.1)
+            seq.append('FIN1')
+
+    async def write(lock):
+        await asyncio.sleep(0)  # PY36 seems to run tasks in the wrong order.
+        async with lock.writer:
+            seq.append('START2')
+            await asyncio.sleep(0.1)
+            seq.append('FIN2')
+
+    lock = RWLock(fast=True)
+    await asyncio.gather(write_wait(lock), write(lock))
+    assert seq == ['READ', 'START2', 'FIN2', 'START1', 'FIN1']
